from __future__ import unicode_literals

from django import forms
from django.contrib.auth.models import User
from django.conf import settings
from courseaffils.models import Course


class CourseAdminForm(forms.ModelForm):
    class Meta:
        model = Course
        exclude = []

    if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER'):
        course_string = settings.COURSEAFFILS_COURSESTRING_MAPPER.widget()

    add_user = forms.CharField(
        required=False,
        widget=forms.Textarea,
        label="Add users to group (one per line)",
        help_text=(
            "Put a [ * ] in front of the username to make "
            "them faculty.  If you add an optional <code>:"
            "&lt;password></code> after the username, the "
            "password on the account will also be set.  <br />"
            "Example: <code>*faculty1:this_is_insecure</code> "
            "creates an instructor account 'faculty1' with the "
            "password 'this_is_insecure' "),
    )

    users_to_remove = forms.ModelMultipleChoiceField(
        required=False,
        widget=forms.CheckboxSelectMultiple,
        queryset=User.objects.none(),
        label="Remove users from group",
    )

    def __init__(self, *args, **kwargs):
        super(forms.ModelForm, self).__init__(*args, **kwargs)
        if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER'):
            self.fields['group'].required = False
        if self.instance.user_set:
            faculty = self.instance.faculty
            ruf = self.fields['users_to_remove']
            ruf.queryset = self.instance.user_set
            ruf.choices = [
                (u.pk,
                 '%s [%s] %s' % (u.get_full_name(), u.username,
                                 '(instructor)' if u in faculty else '')
                 )
                for u in self.instance.user_set.all()
            ]

    def clean(self):
        if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER')  \
                and self.cleaned_data.get('course_string', False):
            m = settings.COURSEAFFILS_COURSESTRING_MAPPER
            stud_grp, fac_grp = m.get_groups(
                self.cleaned_data['course_string'])
            if fac_grp:
                self.cleaned_data['faculty_group'] = fac_grp
            self.cleaned_data['group'] = stud_grp
        elif not self.cleaned_data['group']:
            msg = 'You must select a group'
            if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER'):
                msg = msg + ' or enter a course string'
                if 'course_string' not in self._errors:
                    self._errors['course_string'] = forms.utils.ErrorList()
                self._errors['course_string'].append(msg)
                if 'course_string' in self.cleaned_data:
                    del self.cleaned_data["course_string"]
            self._errors['group'] = forms.utils.ErrorList([msg])
            if 'group' in self.cleaned_data:
                self.cleaned_data["group"]
            raise forms.ValidationError(msg)

        group = self.cleaned_data['group']
        if Course.objects.filter(group=group).exclude(pk=self.instance.pk):
            c = Course.objects.filter(group=group).exclude(
                pk=self.instance.pk).first()
            msg = 'The group "{}" you selected is already associated ' + \
                  'with another course: "{}"'
            msg = msg.format(group.name, c.title)
            self._errors['group'] = forms.utils.ErrorList([msg])
            raise forms.ValidationError(msg)

        # run here, so the cleaned group from above can be used
        self._clean_add_user()

        return self.cleaned_data

    def clean_users_to_remove(self):
        users = self.cleaned_data['users_to_remove']
        if self.instance.group_id:
            group = self.instance.group
            fgroup = self.instance.faculty_group

            for user in users:
                user.groups.remove(group)
                user.groups.remove(fgroup)

        return users

    def _clean_add_user(self):
        """run in clean() so we can process users after course is created
        from course-string
        """
        usernames = self.cleaned_data['add_user']
        if not usernames:
            return

        # take it from here, in case instance is not yet created
        group = self.cleaned_data['group']
        for line in usernames.split('\n'):
            clean_line = line.strip().rstrip().split(':')
            username = clean_line[0]
            password = clean_line[1] if len(clean_line) > 1 else False
            also_faculty = False
            if username.startswith('*'):
                username = username[1:]
                also_faculty = True
            if username:
                try:
                    user = User.objects.get(username=username)
                except User.DoesNotExist:
                    user = User(username=username)
                    if password:
                        user.set_password(password)
                    else:
                        user.set_unusable_password()
                    user.save()
                user.groups.add(group)
                if also_faculty and self.cleaned_data['faculty_group']:
                    user.groups.add(self.cleaned_data['faculty_group'])
        return usernames


class CourseCreateForm(forms.ModelForm):
    class Meta:
        model = Course
        fields = ['title', 'group', 'faculty_group']

    term = forms.ChoiceField(
        choices=((1, 'Spring'), (2, 'Summer'), (3, 'Fall')))
    year = forms.CharField(required=False, min_length=4, max_length=4)
    days = forms.CharField(
        required=False, max_length=7, help_text='e.g. "MTWRF"')
    starttime = forms.TimeField(required=False, help_text='format: HH:MM')
    endtime = forms.TimeField(required=False, help_text='format: HH:MM')

    def save(self, *args, **kwargs):
        r = super(CourseCreateForm, self).save(*args, **kwargs)
        # Add CourseInfo from the fields
        term = self.cleaned_data.get('term')
        year = self.cleaned_data.get('year')
        days = self.cleaned_data.get('days')
        starttime = self.cleaned_data.get('starttime')
        endtime = self.cleaned_data.get('endtime')

        r.info.term = term
        r.info.year = year
        r.info.days = days
        r.info.starttime = starttime
        r.info.endtime = endtime
        r.info.save()

        return r



from __future__ import unicode_literals

from django import template
from courseaffils.lib import get_public_name
from courseaffils.views import get_courses_for_instructor

register = template.Library()


def chunk(chunkable, size=2):
    chunkable = list(chunkable)
    chunked = []
    sublist = []
    while len(chunkable):
        for i in range(size):
            sublist.append(chunkable.pop(0))
        chunked.append(sublist)
        sublist = []
    return chunked


class TemplateTagNode(template.Node):

    noun_for = {'by': 'who'}

    @classmethod
    def error_msg(cls):
        parts = []
        for preposition, noun in cls.noun_for.items():
            parts.append("%s <%s>" % (preposition, noun))
        parts = ' '.join(parts)

        msg = """Invalid tag syntax "%s"
Syntax: %s """ + parts + """ as <varname>"""
        return msg

    @classmethod
    def process_tag(cls, parser, token):
        words = token.split_contents()
        tag_name, words = words[0], words[1:]

        _as, varname = words[-2:]
        words = words[:-2]

        if _as != 'as':
            raise (
                template.TemplateSyntaxError,
                cls.error_msg() % (token.contents, tag_name))

        words = chunk(words)
        if len(words) != len(cls.noun_for):
            raise (
                template.TemplateSyntaxError,
                cls.error_msg() % (token.contents, tag_name))

        kw = {}
        for phrase in words:
            preposition, noun = phrase
            if preposition not in cls.noun_for \
                    or cls.noun_for[preposition] in kw:
                raise (
                    template.TemplateSyntaxError,
                    cls.error_msg() % (token.contents, tag_name))
            kw[cls.noun_for[preposition]] = noun

        return cls(varname, **kw)

    def __init__(self, varname, **kw):
        self.varname = varname
        self.vars = {}
        for key, var in kw.items():
            self.vars[key] = template.Variable(var)

    def render(self, context):
        vars = dict(self.vars)
        for key, var in vars.items():
            vars[key] = var.resolve(context)

        context[self.varname] = self.execute_query(**vars)
        return ''

    def execute_query(self, **kw):
        return ''


class GetCourses(TemplateTagNode):
    noun_for = {"for": "user"}

    def __init__(self, varname, user):
        TemplateTagNode.__init__(self, varname, user=user)

    def execute_query(self, user):
        from courseaffils.views import get_courses_for_user
        return get_courses_for_user(user)


register.tag('get_courses', GetCourses.process_tag)


@register.simple_tag
def get_instructor_courses(user):
    return get_courses_for_instructor(user)


class CourseRole(TemplateTagNode):
    noun_for = {"for": "user", "in": "course"}

    def __init__(self, varname, user, course):
        TemplateTagNode.__init__(self, varname, user=user, course=course)

    def execute_query(self, user, course):
        if not course:
            return "no-course"
        elif course.is_true_faculty(user):
            return "instructor"
        elif course.is_true_member(user):
            return "student"
        else:
            return "non-member"


register.tag('course_role', CourseRole.process_tag)


class PublicName(TemplateTagNode):
    noun_for = {"for": "user"}  # unnecessary, but here for documentation

    def __init__(self, varname, user, truncate=None):
        self.truncate = truncate
        TemplateTagNode.__init__(self, varname, user=user)

    def render(self, context):
        user_s = self.vars['user'].resolve(context)
        if self.truncate is not None:
            user_s = user_s[:self.truncate]
        return get_public_name(user_s, context['request'])

    @classmethod
    def process_tag(cls, parser, token):
        "words = (<tagname>,'for',<varname> [truncate <max_users>])"
        words = token.split_contents()
        if len(words) < 4:
            return cls(words[2], user=words[2])
        else:
            return cls(words[2], user=words[2], truncate=int(words[4]))


register.tag('public_name', PublicName.process_tag)



from __future__ import unicode_literals

import re
try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen
from time import strptime, strftime


class CourseStringMapper:
    @staticmethod
    def widget():
        from django import forms
        w = forms.CharField(
            required=False,
            widget=forms.TextInput,
            help_text=(
                'Alternate to choosing a group. '
                'Enter the section key from the directory of courses. '
                'e.g. 20101SCNC1000F001 (which is yyyytddddnnnnLsss where '
                'y=year, t=term (with "1" for Spring and '
                '"3" for Fall), '
                'd=dept, n=course number, '
                'L=course letter, s=section). You can also enter the course-'
                'string in the format the group names appear for another '
                'course (e.g. "t1.y2010.s001.cf1000.scnc.st.course:'
                'columbia.edu" ). '
                '<br /><br /> '
                'This will create a group that automatically ties '
                'registered students '
                'to the class upon login. '
                '<br /><br /> '
                'If the course is not found in the '
                '<a href="http://www.columbia.edu/cu/bulletin/uwb/">'
                'Directory of Classes</a>, then it will still create '
                'a course. '
                'This is a GOOD WAY to create a course that is not '
                'associated '
                'with a directory. Alternatively, click the (+) '
                'signs next '
                'to "Group" and "Faculty group" fields and create groups '
                'manually, before clicking save. '
                '<br /> '
                'If you accidentally entered the wrong course '
                'string, blank out '
                'the groups (course and faculty) and type in '
                'the string again.')
        )

        def clean(sectionkey):
            if sectionkey:
                class_info = SectionkeyTemplate.to_dict(sectionkey)
                if not class_info:
                    raise forms.ValidationError('Invalid course string.')
                else:
                    return sectionkey
        w.clean = clean
        return w

    @staticmethod
    def get_groups(sectionkey):
        from django.contrib.auth.models import Group
        section_dict = SectionkeyTemplate.to_dict(sectionkey)
        if not section_dict:
            section_dict = WindTemplate.to_dict(sectionkey)

        if section_dict:
            stud_grp_name = WindTemplate.to_string(section_dict)
            fac_grp_name = WindTemplate.to_string(
                dict(section_dict, member='fc'))

            stud_grp, created = Group.objects.get_or_create(name=stud_grp_name)
            fac_grp, created = Group.objects.get_or_create(name=fac_grp_name)
            return stud_grp, fac_grp
        else:
            return None, None

    @staticmethod
    def get_course_info(course_dict):
        directory_link = DirectoryLinkTemplate.to_string(course_dict)
        response = urlopen(directory_link).read()
        return DirectoryPageTemplate.to_dict(response)

    @staticmethod
    def on_create(course):
        course_dict = WindTemplate.to_dict(course.group.name)
        from courseaffils.models import CourseInfo

        info, created = CourseInfo.objects.get_or_create(course=course)
        if course_dict:
            info.year = int(course_dict['year'])
            info.term = int(course_dict['term'])
        try:
            info_from_web = CourseStringMapper.get_course_info(
                WindTemplate.to_dict(course.group.name))

            for val in info_from_web:
                if val == 'days':
                    info.days = info_from_web[val]
                elif val in ('starttime', 'endtime'):
                    tm = strptime(info_from_web[val], '%I:%M%p')
                    setattr(info, val, strftime('%H:%M', tm))
                else:
                    course.coursedetails_set.get_or_create(
                        name=val,
                        value=info_from_web[val],
                        course=course)
        except (TypeError, KeyError, ValueError):
            # oh well, couldn't get extra data.
            # maybe because it's a fake course string or not a proper course
            pass
        info.save()

    @staticmethod
    def course_slug(course, attempt=0):
        "returns a slug for the course, with higher resolution for attempt > 0"
        class_info = WindTemplate.to_dict(course.group.name)
        slug = None
        if class_info and 'number' in class_info:
            attempts = {
                0: "CU%s%s" % (class_info['dept'],
                               class_info['number'], ),
                1: "CU%s%s_%s" % (class_info['dept'],
                                  class_info['number'],
                                  class_info['section'],),
            }
            slug = attempts[attempt]
        else:
            slug = re.sub(r' ', '_', course.title)
        return re.sub(r'\W', '', slug)

    @staticmethod
    def to_string(cdict):
        return WindTemplate.to_string(cdict)

    @staticmethod
    def to_dict(wind_string):
        return WindTemplate.to_dict(wind_string)


class WindTemplate:
    example = 't3.y2007.s001.cw3956.engl.fc.course:columbia.edu'

    @staticmethod
    def to_string(cdict):
        cdict['member'] = cdict.get('member', 'st')
        return 't%s.y%s.s%s.c%s%s.%s.%s.course:columbia.edu' % (
            cdict['term'],
            cdict['year'],
            cdict['section'],
            cdict['letter'].lower(),
            cdict['number'],
            cdict['dept'].lower(),
            cdict['member'].lower(),
        )

    @staticmethod
    def to_dict(wind_string):
        wind_match = re.match(
            (r't(?P<term>\d).y(?P<year>\d{4}).s(?P<section>\w{3})'
             r'.c(?P<letter>\w)(?P<number>\w{4})'
             r'.(?P<dept>[\w&]{4}).(?P<member>\w\w).course:columbia.edu'),
            wind_string)
        if wind_match:
            return wind_match.groupdict()


class SectionkeyTemplate:
    example = '20101SCNC1000F001'  # and 20103MIMD036PN004

    @staticmethod
    def to_dict(sectionkey):
        key_match = re.match(
            (r'(?P<year>\d{4})(?P<term>\d)(?P<dept>[\w&]{4})'
             r'(?P<number>\w{4})(?P<letter>\w)(?P<section>\w{3})'),
            sectionkey)
        if key_match:
            return key_match.groupdict()


class DirectoryLinkTemplate:
    example = ('http://www.columbia.edu/cu/bulletin/'
               'uwb/subj/SCNC/C1100-20101-003/')

    @staticmethod
    def to_string(cdict):
        return (
            'http://www.columbia.edu/cu/bulletin/'
            'uwb/subj/%s/%s%s-%s%s-%s/') % (
                cdict['dept'].upper(),
                cdict['letter'].upper(),
                cdict['number'],
                cdict['year'],
                cdict['term'],
                cdict['section'], )


class CanvasTemplate:
    example = 'SDEVW2300_001_2011_1'

    @staticmethod
    def to_dict(affil_string):
        affil_match = re.match(
            (r'(?P<dept>[\w&]{4})(?P<letter>\w)'
             r'(?P<number>\w{4})_(?P<section>\d{3})_'
             r'(?P<year>\d{4})_(?P<term>\d)'),
            affil_string)
        if affil_match:
            return affil_match.groupdict()


class AffilTemplate:
    example = 'CUcourse_SDEVW2300_001_2011_1'

    @staticmethod
    def to_dict(affil_string):
        affil_match = re.match(
            (r'CUcourse_(?P<dept>[\w&]{4})(?P<letter>\w)'
             r'(?P<number>\w{4})_(?P<section>\d{3})_'
             r'(?P<year>\d{4})_(?P<term>\d)'),
            affil_string)
        if affil_match:
            return affil_match.groupdict()


class DirectoryPageTemplate:
    @staticmethod
    def to_dict(html_page):
        ret_val = {}
        look_for = (
            r'Call Number</td>[^>]*>(?P<call_number>\d*)</td>',
            r'Location</td>[^>]*>(?P<times>[^<]*)<br>(?P<location>[^<]*)</td>',
            r'Points</td>[^>]*>(?P<points>[\d\-.]*)</td>',
            r'Campus</td>[^>]*>(?P<campus>[\w\s]*)</td>',
            r'Instructor</td>[^>]*>(?P<instructor>[^<]*)</td>',
            # 'approvals_required', 'type', 'enrollment',
            # 'max_enrollment', 'status',
            # 'title', 'open_to', 'note', 'sectionkey',
        )
        for reg in look_for:
            m = re.search(reg, html_page)
            if m:
                ret_val.update(m.groupdict())

        if 'times' in ret_val:
            m = re.match(
                (r'(?P<days>\w*)\s*(?P<starttime>[\d:apm]*)\-'
                 r'(?P<endtime>[\d:apm]*)'),
                ret_val['times'])
            if m:
                ret_val.update(m.groupdict())

        return ret_val


class HashTagTemplate:
    example = '#CUviet1201'

    @staticmethod
    def to_string(cdict):
        return '#CU%s%s' % (
            cdict['dept'].lower(),
            cdict['number'],
        )



from __future__ import unicode_literals

from django.conf.urls import url
from courseaffils import views


urlpatterns = [
    url(r'^select_course/$',
        views.CourseListView.as_view(),
        name='select_course'),
    url(r'^course/create/$',
        views.CourseCreateView.as_view(),
        name='create_course'),
]



from django.utils import timezone


SPRING = 1
SUMMER = 2
FALL = 3


def get_current_term():
    """Find out what semester we're currently in.

    Return values are:
    1 - Spring
    2 - Summer
    3 - Fall
    """
    today = timezone.now().date()

    # Assume that schools start summer semester in the second half
    # of May or later.
    if today.month <= 4 or (today.month == 5 and today.day <= 15):
        return SPRING
    elif today.month <= 8:
        return SUMMER
    else:
        return FALL



from __future__ import unicode_literals

from django.contrib import admin
from courseaffils.models import (
    Affil,
    Course, CourseSettings, CourseInfo, CourseDetails
)
from courseaffils.forms import CourseAdminForm


class CourseAdmin(admin.ModelAdmin):
    form = CourseAdminForm

    search_fields = ('title',)
    list_display = ('title', 'id',)
    change_form_template = "courseaffils/admin_change_form.html"


class AffilAdmin(admin.ModelAdmin):
    model = Affil
    list_display = ['name', 'user']


admin.site.register(Course, CourseAdmin)
admin.site.register(Affil, AffilAdmin)
admin.site.register(CourseSettings)
admin.site.register(CourseInfo)
admin.site.register(CourseDetails)



from __future__ import unicode_literals

from django.test import TestCase, override_settings
from django.utils.encoding import smart_text
from courseaffils.columbia import CourseStringMapper
from courseaffils.models import Course, CourseSettings
from courseaffils.models import CourseInfo, CourseAccess
from courseaffils.tests.factories import (
    AffilFactory, CourseFactory, UserFactory
)
from django.contrib.auth.models import Group, User
from freezegun import freeze_time


class UserTests(TestCase):
    def setUp(self):
        self.u = UserFactory()

    def test_is_valid_from_factory(self):
        self.u.full_clean()


class CourseTests(TestCase):
    def setUp(self):
        self.c = CourseFactory()

    def test_is_valid_from_factory(self):
        self.c.full_clean()
        self.c.info.full_clean()


class ModelsSimpleTest(TestCase):
    """ test some basic happy paths through the models """
    def setUp(self):
        self.student_group = Group.objects.create(name="studentgroup")
        self.faculty_group = Group.objects.create(name="facultygroup")
        self.student = User.objects.create(username="student")
        self.faculty = User.objects.create(username="faculty")
        self.student.groups.add(self.student_group)
        self.faculty.groups.add(self.faculty_group)
        self.c = Course.objects.create(
            group=self.student_group,
            title="test course",
            faculty_group=self.faculty_group)

    def tearDown(self):
        self.c.delete()
        self.student_group.delete()
        self.faculty_group.delete()
        self.student.delete()
        self.faculty.delete()

    def test_str(self):
        self.assertEqual(smart_text(self.c), "test course")

    def test_members(self):
        self.assertIn(self.student, self.c.members)

    def test_students(self):
        self.assertIn(self.student, self.c.students)

    def test_students_no_faculty(self):
        """ need this to get complete coverage on the .students()
        method, handling the case where there is no faculty group"""
        second_student_group = Group.objects.create(name="2nd student group")
        self.student.groups.add(second_student_group)
        nf_course = Course.objects.create(
            group=second_student_group,
            title="course with no faculty group")
        self.assertIn(self.student, nf_course.students)
        nf_course.delete()
        second_student_group.delete()

    def test_faculty(self):
        self.assertIn(self.faculty, self.c.faculty)

    def test_faculty_no_faculty(self):
        """ need this to get complete coverage on the .faculty()
        method, handling the case where there is no faculty group"""
        second_student_group = Group.objects.create(name="2nd student group")
        self.student.groups.add(second_student_group)
        nf_course = Course.objects.create(
            group=second_student_group,
            title="course with no faculty group")
        self.assertEqual(nf_course.faculty, ())
        nf_course.delete()
        second_student_group.delete()

    def test_user_set(self):
        self.assertIn(self.student, self.c.user_set.all())

    def test_is_faculty(self):
        self.assertTrue(self.c.is_faculty(self.faculty))
        self.assertFalse(self.c.is_faculty(self.student))

    def test_is_member(self):
        self.assertTrue(self.c.is_member(self.student))

    def test_is_true_member(self):
        self.assertTrue(self.c.is_true_member(self.student))

    def test_is_true_member_nonexistant(self):
        class StubUser(object):
            id = -1
        u = StubUser()
        self.assertFalse(self.c.is_true_member(u))

    def test_faculty_filter(self):
        # TODO: I don't actually know what to do with this
        # I don't know what the faculty_filter is for
        # and it looks like it *ought* to not work,
        # since it's filtering on an "author" field that
        # shouldn't exist (or at least, I don't know where it's
        # supposed to be used). And I can't find a usage of this
        # anywhere else in the courseaffils code
        # for now, just execute it to at least get coverage
        self.c.faculty_filter

    def test_default_slug(self):
        self.assertEqual(self.c.default_slug(), "test_course")

    def test_slug(self):
        # TODO: mock out django settings for this one
        # since it actually can change the behavior
        self.assertEqual(self.c.slug(), "test_course")

    def test_details(self):
        # shouldn't be any to start with
        self.assertEqual(self.c.details(), dict())
        self.c.add_detail("foo", "bar")
        self.assertIn("foo", self.c.details())
        self.assertEqual(self.c.details()["foo"].value, "bar")
        self.assertEqual(self.c.get_detail("foo", default="not bar"), "bar")
        self.assertEqual(self.c.get_detail(
            "nonexistant",
            default="a default value"), "a default value")

        self.assertEqual(smart_text(self.c.details()["foo"]),
                         "(test course) foo: bar")

        # update
        self.c.add_detail("foo", "baz")
        self.assertEqual(self.c.details()["foo"].value, "baz")

    def test_coursesettings(self):
        cs = CourseSettings.objects.create(
            course=self.c,
            custom_headers="some headers")
        self.assertEqual(smart_text(cs), "Settings for test course")

    def test_courseinfo(self):
        # current behavior is that a CourseInfo object
        # is created automatically for each course
        # by a hook elsewhere. so verify that.
        self.assertNotEqual(CourseInfo.objects.all().count(), 0)

        self.assertEqual(smart_text(self.c.info),
                         'test course () None None-None')

        self.c.info.year = 2013
        self.c.info.term = 1
        self.c.info.days = "MWF"
        self.c.info.save()
        self.assertEqual(smart_text(self.c.info),
                         'test course (Spring 2013) MWF None-None')

        self.assertEqual(self.c.info.time(), 'MWF')

        self.assertEqual(self.c.info.display(), "Spring 2013 MWF None-None")

    def test_courseaccess(self):
        """ what is this? """

        # respond() should do nothing
        CourseAccess.respond("foo")

        class StubRequest(object):
            POST = dict()
            GET = dict()

        self.assertFalse(CourseAccess.allowed(StubRequest()))


@override_settings(COURSEAFFILS_COURSESTRING_MAPPER=CourseStringMapper)
class AffilTest(TestCase):
    def setUp(self):
        self.aa = AffilFactory(
            name='t1.y2016.s001.cf1002.scnc.st.course:columbia.edu')

    def test_is_valid_from_factory(self):
        self.aa.full_clean()

    def test_str(self):
        self.assertEqual(smart_text(self.aa), self.aa.name)

    def test_past_present_future(self):
        with freeze_time('2012-01-14'):
            self.assertEqual(self.aa.past_present_future, 1)
        with freeze_time('2015-01-14'):
            self.assertEqual(self.aa.past_present_future, 1)
        with freeze_time('2015-08-15'):
            self.assertEqual(self.aa.past_present_future, 1)
        with freeze_time('2015-12-15'):
            self.assertEqual(self.aa.past_present_future, 1)
        with freeze_time('2016-01-14'):
            self.assertEqual(self.aa.past_present_future, 0)
        with freeze_time('2016-04-26'):
            self.assertEqual(self.aa.past_present_future, 0)
        with freeze_time('2016-06-04'):
            self.assertEqual(self.aa.past_present_future, -1)
        with freeze_time('2017-01-14'):
            self.assertEqual(self.aa.past_present_future, -1)
        with freeze_time('2017-03-30'):
            self.assertEqual(self.aa.past_present_future, -1)
        with freeze_time('2017-10-03'):
            self.assertEqual(self.aa.past_present_future, -1)
        with freeze_time('2017-12-03'):
            self.assertEqual(self.aa.past_present_future, -1)

        aa = AffilFactory(
            name='t2.y2016.s001.cf1002.scnc.st.course:columbia.edu')
        with freeze_time('2016-01-14'):
            self.assertEqual(aa.past_present_future, 1)
        with freeze_time('2016-04-26'):
            self.assertEqual(aa.past_present_future, 1)
        with freeze_time('2016-08-12'):
            self.assertEqual(aa.past_present_future, 0)
        with freeze_time('2017-12-03'):
            self.assertEqual(aa.past_present_future, -1)

        aa = AffilFactory(
            name='t3.y2016.s001.cf1002.scnc.st.course:columbia.edu')
        with freeze_time('2016-01-14'):
            self.assertEqual(aa.past_present_future, 1)
        with freeze_time('2016-04-26'):
            self.assertEqual(aa.past_present_future, 1)
        with freeze_time('2016-10-02'):
            self.assertEqual(aa.past_present_future, 0)
        with freeze_time('2017-12-03'):
            self.assertEqual(aa.past_present_future, -1)

        aa = AffilFactory(name='bogus affil string')
        self.assertEqual(aa.past_present_future, None)

    def test_courseworks_name(self):
        self.assertEqual(
            self.aa.courseworks_name,
            'CUcourse_SCNCf1002_001_2016_1')

    def test_coursedirectory_name(self):
        self.assertEqual(
            self.aa.coursedirectory_name,
            '20161SCNC1002F001')

    def test_shortname(self):
        self.assertEqual(self.aa.shortname, 'SCNC1002')

    def test_is_faculty(self):
        self.assertEqual(self.aa.is_faculty, False)

        aa = AffilFactory(
            name='t1.y2016.s002.cf1402.scnc.fc.course:columbia.edu')
        self.assertEqual(aa.is_faculty, True)

    def test_get_course(self):
        self.assertIsNone(self.aa.get_course())



from __future__ import unicode_literals

from courseaffils.columbia import AffilTemplate
from courseaffils.columbia import CourseStringMapper
from courseaffils.columbia import DirectoryPageTemplate
from courseaffils.columbia import HashTagTemplate, CanvasTemplate
from courseaffils.columbia import SectionkeyTemplate
from courseaffils.columbia import WindTemplate
from django.test import TestCase


class DummyRequest(object):
    COOKIES = dict()


class ColumbiaSimpleTest(TestCase):

    def test_hashtagtemplate(self):
        self.assertEqual(
            HashTagTemplate.to_string(dict(dept="FOO", number="five")),
            "#CUfoofive")

    def test_directorytagtemplate(self):
        # searching an empty page should return an empty dict
        self.assertEqual(DirectoryPageTemplate.to_dict(""), dict())

        self.assertEqual(DirectoryPageTemplate.to_dict(
            """Points</td><td>5</td>"""), dict(points="5"))

        self.assertEqual(
            DirectoryPageTemplate.to_dict(
                'Location</td><td>MW 9am-10<br>Lewisohn 123</td>'
            ),
            dict(
                location="Lewisohn 123",
                times="MW 9am-10",
                days="MW",
                starttime="9am",
                endtime="10",))

    def test_affiltemplate(self):
        self.assertEqual(
            AffilTemplate.to_dict(AffilTemplate.example),
            dict(
                dept="SDEV",
                letter="W",
                number="2300",
                section="001",
                year="2011",
                term="1",
            )
        )

    def test_directorylinktemplate(self):
        pass

    def test_sectionkeytemplate(self):
        example1 = '20101SCNC1000F001'  # and 20103MIMD036PN004
        self.assertEqual(
            SectionkeyTemplate.to_dict(example1),
            dict(
                year="2010",
                term="1",
                dept="SCNC",
                number="1000",
                letter="F",
                section="001",))

        # a newstyle sw section key, w/ a leading letter for the section

        example2 = '20153SOCW0006TD21'
        self.assertEqual(
            SectionkeyTemplate.to_dict(example2),
            dict(
                year="2015",
                term="3",
                dept="SOCW",
                number="0006",
                letter="T",
                section="D21",))

    def test_windtemplate(self):
        example = 't3.y2007.s001.cw3956.engl.fc.course:columbia.edu'
        # round-trip it
        self.assertEqual(
            WindTemplate.to_string(WindTemplate.to_dict(example)),
            example)

        example = 't3.y2007.sd21.cw3956.engl.fc.course:columbia.edu'
        # round-trip it
        self.assertEqual(
            WindTemplate.to_string(WindTemplate.to_dict(example)),
            example)
        self.assertEqual(
            CourseStringMapper.to_string(CourseStringMapper.to_dict(example)),
            example)

    def test_csmapper_course_slug(self):
        class StubGroup(object):
            name = 't3.y2007.s001.cw3956.engl.fc.course:columbia.edu'

        class StubCourse(object):
            def __init__(self):
                self.group = StubGroup()
        self.assertEqual(
            CourseStringMapper.course_slug(StubCourse()),
            'CUengl3956')

    def test_csmapper_widget(self):
        self.assertEqual(
            CourseStringMapper.widget().clean('20101SCNC1000F001'),
            '20101SCNC1000F001')

    def test_csmapper_get_groups(self):
        s, f = CourseStringMapper.get_groups('20101SCNC1000F001')
        self.assertIsNotNone(s)
        self.assertIsNotNone(f)
        s, f = CourseStringMapper.get_groups(WindTemplate.example)
        self.assertIsNotNone(s)
        self.assertIsNotNone(f)

        s, f = CourseStringMapper.get_groups("something completely invalid")
        self.assertIsNone(s)
        self.assertIsNone(f)

    def test_csmapper_get_course_info(self):
        # this one makes an HTTP request to the CU DOC site
        # so it's not really appropriate for unit testing
        pass

    def test_canvas_mapper(self):
        d = CanvasTemplate.to_dict('SOCWT7113_010_2017_3')
        self.assertEquals(d['term'], u'3')
        self.assertEquals(d['section'], u'010')
        self.assertEquals(d['number'], u'7113')
        self.assertEquals(d['dept'], u'SOCW')
        self.assertEquals(d['letter'], u'T')
        self.assertEquals(d['year'], u'2017')

        s = WindTemplate.to_string(d)
        self.assertEquals(s,
                          't3.y2017.s010.ct7113.socw.st.course:columbia.edu')



from __future__ import unicode_literals

from courseaffils.columbia import CourseStringMapper


SECRET_KEY = 'fake-key'
MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
]
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            # insert your TEMPLATE_DIRS here
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'courseaffils',
    'django_markwhat',
    'django_jenkins',
)

PROJECT_APPS = [
    'courseaffils',
]
COVERAGE_EXCLUDES_FOLDERS = ['migrations']

COURSEAFFILS_EXEMPT_PATHS = (
    '/accounts/',
    '/static/',
    '/site_media/',
    '/docs/',
    '/admin/',
    '/registration/',
    '/favicon.ico',
    '/smoketest/',
)
COURSEAFFILS_COURSESTRING_MAPPER = CourseStringMapper
ROOT_URLCONF = 'courseaffils.tests.urls'

# Django replaces this, but it still wants it. *shrugs*
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
        'HOST': '',
        'PORT': '',
        'USER': '',
        'PASSWORD': '',
    }
}



from __future__ import unicode_literals

import factory
from courseaffils.models import Affil, Course
from django.contrib.auth.models import User, Group


class UserFactory(factory.DjangoModelFactory):
    class Meta:
        model = User

    username = factory.Sequence(lambda n: 'user%03d' % n)
    password = factory.PostGenerationMethodCall('set_password', 'test')
    email = factory.LazyAttribute(lambda u: '%s@example.com' % u.username)


class GroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = Group

    name = factory.Sequence(lambda n: 'Example Group %d' % n)


class CourseFactory(factory.DjangoModelFactory):
    class Meta:
        model = Course

    title = factory.Sequence(lambda n: 'Example Course %d' % n)
    group = factory.SubFactory(GroupFactory)
    faculty_group = factory.SubFactory(GroupFactory)


class AffilFactory(factory.DjangoModelFactory):
    class Meta:
        model = Affil

    name = factory.Sequence(
        lambda n: 't1.y2016.s001.cf100%d.scnc.st.course:columbia.edu' % n)
    user = factory.SubFactory(UserFactory)



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.template.loader import get_template
from django.http import Http404
from courseaffils.models import Affil, Course


def faculty_courses_for_user(user):
    """Return the Courses that the given user is an instructor for.

    Returns a QuerySet.
    """
    return Course.objects.filter(faculty_group__in=user.groups.all())


def is_faculty(user):
    """Return True if the given user is a faculty member on any courses."""
    return Affil.objects.filter(user=user).exists() or \
        faculty_courses_for_user(user).exists()


def users_in_course(course):
    return User.objects.filter(groups=course.group)


def in_course(username, group_or_course):
    group = getattr(group_or_course, 'group', group_or_course)
    try:
        return group.user_set.get(username=username)
    except User.DoesNotExist:
        return False


def in_course_or_404(username, group_or_course):
    "Supports either the course-group or course as second arg"
    group = getattr(group_or_course, 'group', group_or_course)
    try:
        return group.user_set.get(username=username)
    except User.DoesNotExist:
        template = get_template('not_in_course.html')
        context = {
            'user': username,
            'course': group,
        }
        response_body = template.render(context)
        raise Http404(response_body)


ANONYMIZE_KEY = 'ccnmtl.courseaffils.anonymize'


def handle_public_name(user, request):
    """guarantees no double-quotes so also json-friendly"""
    if 'ANONYMIZE' in request.COOKIES:
        request.__dict__.setdefault('scrub_names', {})
        request.scrub_names[user] = user.id
        return 'User Name_%d' % user.id
    else:
        return (user.get_full_name() or user.username).replace('"', "'")


def get_public_name(user_s, request):
    if hasattr(user_s, 'is_anonymous'):
        return handle_public_name(user_s, request)
    else:
        # for attribution lists, and such
        return ', '.join([handle_public_name(u, request) for u in user_s])


# AUTO_COURSE_SELECT is a dictionary that can be populated by other
# django apps in their views.py.  The KEY should be the view
# and the VALUE should be a function that takes the same arguments
# (except request) as the view, but returns the corresponding course
# If no course is possible it should return None
# SAMPLE VIEW CODE:
# from courseaffils.lib import AUTO_COURSE_SELECT
# def my_view(request, obj_id):
#    ....
# def my_view_courselookup(obj_id=None):
#    if obj_id:
#        return My.objects.get(pk=obj_id).course
# AUTO_COURSE_SELECT[my_view] = my_view_courselookup
#
AUTO_COURSE_SELECT = {}



from __future__ import unicode_literals

from courseaffils.middleware import CourseManagerMiddleware
from courseaffils.middleware import already_selected_course
from courseaffils.middleware import is_anonymous_path
from courseaffils.models import Course
from django.contrib.auth.models import Group, User
from django.http.response import Http404
from django.test import TestCase


class StubRequest(object):
    COOKIES = dict()
    GET = dict()
    POST = dict()
    environ = dict()

    def __init__(self, c):
        self.path = "/foo/bar"
        if c:
            self.session = {'ccnmtl.courseaffils.course': c}
        else:
            self.session = dict()

    def get_full_path(self):
        return self.path


class StubResponse(object):
    content = ""


class MiddlewareSimpleTest(TestCase):
    """
    These are really bad. courseaffils.middleware
    does a bunch of inspecting the settings
    directly, so it's hard to make tests that
    run independently of the application that
    is including it. Eg, for the moment,
    I run these tests from within mvsim,
    so it's testing those settings. We can
    improve this once we have a dedicated
    test harness for courseaffils, it will
    still be messy.
    """
    def setUp(self):
        self.student_group = Group.objects.create(name="studentgroup")
        self.faculty_group = Group.objects.create(name="facultygroup")
        self.student = User.objects.create(username="student",
                                           last_name="long enough")
        self.faculty = User.objects.create(username="faculty")
        self.student.groups.add(self.student_group)
        self.faculty.groups.add(self.faculty_group)
        self.c = Course.objects.create(
            group=self.student_group,
            title="test course",
            faculty_group=self.faculty_group)

    def test_is_anonymous_path(self):
        assert is_anonymous_path("/favicon.ico")
        assert not is_anonymous_path("/game")
        assert is_anonymous_path("/static/")

    def test_already_selected_course(self):
        assert already_selected_course(StubRequest(True))
        assert not already_selected_course(StubRequest(False))

    def test_cmm_process_response(self):
        c = CourseManagerMiddleware()
        assert c.process_response(StubRequest(True), "foo") == "foo"

    def test_cmm_process_response_anon(self):
        c = CourseManagerMiddleware()
        r = StubRequest(self.c)
        r.user = self.student
        r.COOKIES['ANONYMIZE'] = True
        r.scrub_names = {self.student: 1}
        resp = StubResponse()
        resp.content = str(self.student.get_full_name())
        assert "User Name" in c.process_response(r, resp).content
        resp.content = str(self.student.get_full_name())
        assert "long enough" not in c.process_response(r, resp).content

    def test_cmm_process_request(self):
        c = CourseManagerMiddleware()
        r = StubRequest(self.c)
        r.user = self.student
        assert c.process_request(r) is None

        r.path = "/favicon.ico"
        assert c.process_request(r) is None

        r = StubRequest(self.c)
        r.user = self.student
        r.GET['unset_course'] = True
        assert c.process_request(r) is None

        r = StubRequest(self.c)
        r.user = self.student
        r.POST['set_course'] = 'studentgroup'
        assert c.process_request(r) is None

        r = StubRequest(self.c)
        r.user = self.student
        r.POST['set_course'] = 'foobarbaz'
        with self.assertRaises(Http404):
            c.process_request(r)

        r = StubRequest(self.c)
        r.user = self.student
        r.POST['set_course'] = 'studentgroup'
        r.GET['next'] = "/foo"
        assert c.process_request(r) is not None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('courseaffils', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='courseinfo',
            name='days',
            field=models.CharField(
                help_text=b'e.g. "MTWRF"', max_length=7,
                null=True, blank=True),
        ),
    ]



from __future__ import unicode_literals

from datetime import time
from courseaffils.models import Course, CourseInfo
from courseaffils.tests.factories import (
    CourseFactory, UserFactory, GroupFactory
)
from courseaffils.tests.mixins import LoggedInFacultyTestMixin
from courseaffils.views import get_courses_for_user, get_courses_for_instructor
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
from django.test import TestCase


class ViewTests(TestCase):

    def test_root(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 404)

    def test_get_courses(self):
        course = CourseFactory()
        student = UserFactory()
        instructor = UserFactory()
        staff = UserFactory(is_staff=True)

        course.group.user_set.add(student)
        course.group.user_set.add(instructor)
        course.faculty_group.user_set.add(instructor)

        # as student
        lst = get_courses_for_user(student)
        self.assertEquals(len(lst), 1)
        self.assertTrue(course in lst)

        lst = get_courses_for_instructor(student)
        self.assertEquals(len(lst), 0)

        # as instructor
        lst = get_courses_for_user(instructor)
        self.assertEquals(len(lst), 1)
        self.assertTrue(course in lst)

        lst = get_courses_for_instructor(instructor)
        self.assertEquals(len(lst), 1)
        self.assertTrue(course in lst)

        # as staff
        lst = get_courses_for_user(staff)
        self.assertEquals(len(lst), 1)
        self.assertTrue(course in lst)

        lst = get_courses_for_instructor(staff)
        self.assertEquals(len(lst), 1)
        self.assertTrue(course in lst)


class CourseListViewTests(TestCase):
    def test_select_course_empty(self):
        response = self.client.get(reverse('select_course'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 0)

    def test_select_course(self):
        u = UserFactory(username='test')
        self.client.login(username='test', password='test')

        c = CourseFactory(title='My Course')

        response = self.client.get(reverse('select_course'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 0)
        response = self.client.get(reverse('select_course'),
                                   {'semester_view': 'sandbox'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            len(response.context['object_list']), 0,
            'Non-staff users shouldn\'t see sandbox courses '
            'they aren\'t part of.')

        u.groups.add(c.group)
        response = self.client.get(reverse('select_course'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 0)
        response = self.client.get(reverse('select_course'),
                                   {'semester_view': 'sandbox'})
        self.assertEqual(
            len(response.context['object_list']), 1,
            'Non-staff users should see sandbox courses '
            'they are grouped with.')
        self.assertEqual(
            response.context['object_list'][0].title,
            'My Course')

    def test_select_course_staff(self):
        UserFactory(username='test', is_staff=True)
        self.client.login(username='test', password='test')

        CourseFactory(title='My Course')

        response = self.client.get(reverse('select_course'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 0)
        response = self.client.get(reverse('select_course'),
                                   {'semester_view': 'sandbox'})
        self.assertEqual(len(response.context['object_list']), 1)
        self.assertEqual(
            response.context['object_list'][0].title,
            'My Course')


class CourseCreateViewFacultyTests(TestCase, LoggedInFacultyTestMixin):
    def test_get(self):
        response = self.client.get(reverse('create_course'))
        self.assertEqual(response.status_code, 200)

    def test_post(self):
        group = GroupFactory()

        self.assertEqual(Course.objects.count(), 0)
        self.assertEqual(CourseInfo.objects.count(), 0)
        response = self.client.post(reverse('create_course'), {
            'title': 'My Title',
            'group': group.pk,
            'faculty_group': group.pk,
            'term': 1,
            'year': 2016,
            'days': 'TR',
            'starttime': time(hour=16),
            'endtime': time(hour=18),
        })
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Course.objects.count(), 1)
        course = Course.objects.first()
        self.assertEqual(course.title, 'My Title')

        self.assertEqual(CourseInfo.objects.count(), 1)
        info = course.info
        self.assertEqual(info, CourseInfo.objects.first())
        self.assertEqual(info.term, 1)
        self.assertEqual(info.year, 2016)
        self.assertEqual(info.days, 'TR')
        self.assertEqual(info.starttime, time(hour=16))
        self.assertEqual(info.endtime, time(hour=18))



from django.test import TestCase
from courseaffils.utils import get_current_term
from freezegun import freeze_time


class TestUtils(TestCase):
    def test_get_current_term(self):
        with freeze_time('2016-01-01'):
            self.assertEqual(get_current_term(), 1)
        with freeze_time('2016-03-16'):
            self.assertEqual(get_current_term(), 1)
        with freeze_time('2016-04-30'):
            self.assertEqual(get_current_term(), 1)
        with freeze_time('2016-05-01'):
            self.assertEqual(get_current_term(), 1)
        with freeze_time('2016-05-20'):
            self.assertEqual(get_current_term(), 2)
        with freeze_time('2016-08-20'):
            self.assertEqual(get_current_term(), 2)
        with freeze_time('2016-09-02'):
            self.assertEqual(get_current_term(), 3)
        with freeze_time('2016-12-12'):
            self.assertEqual(get_current_term(), 3)



from __future__ import unicode_literals

from django.contrib.auth.models import Group
from django.conf import settings

auto_groups = getattr(settings, "COURSEAFFIL_AUTO_MAP_GROUPS", [])


def auto_group_mapper(sender, **kwargs):
    for group in auto_groups:
        group = Group.objects.get_or_create(name=group)[0]
        sender.groups.add(group)
        sender.save()


class AutoGroupWindMapper:
    """
    maps all wind-based users to a fixed set of groups on login
    (autovivifying if necessary)
    """

    def map(self, user, affils):
        for group in auto_groups:
            group = Group.objects.get_or_create(name=group)[0]
            user.groups.add(group)
            user.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-23 12:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('courseaffils', '0003_auto_20160429_1353'),
    ]

    operations = [
        migrations.AddField(
            model_name='course',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AddField(
            model_name='course',
            name='updated_at',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
    ]



# flake8: noqa
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Course',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=1024)),
                ('faculty_group', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, related_name='faculty_of', blank=True, to='auth.Group', null=True)),
                ('group', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='CourseDetails',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(help_text=b"type of data. Useful ones are 'instructor', 'semester', 'url', 'campus', 'times', 'call_number'", max_length=64)),
                ('value', models.CharField(help_text=b"The name's value for the course.", max_length=1024)),
                ('course', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courseaffils.Course')),
            ],
            options={
                'verbose_name_plural': 'Course Details',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='CourseInfo',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('year', models.IntegerField(null=True, blank=True)),
                ('term', models.IntegerField(blank=True, null=True, choices=[(1, b'Spring'), (2, b'Summer'), (3, b'Fall')])),
                ('starttime', models.TimeField(null=True, blank=True)),
                ('endtime', models.TimeField(null=True, blank=True)),
                ('days', models.CharField(max_length=7, null=True, blank=True)),
                ('course', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='info', to='courseaffils.Course')),
            ],
            options={
                'verbose_name_plural': 'Course Info',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='CourseSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('custom_headers', models.TextField(help_text=b'Replaces main.css link in header.  You need to add this as full HTML (&lt;link rel="stylesheet" href="...." />) but the advantage is you can add custom javascript here, too.', null=True, blank=True)),
                ('course', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='settings', to='courseaffils.Course')),
            ],
            options={
                'verbose_name_plural': 'Course Settings',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

import json
from courseaffils.utils import get_current_term
from courseaffils.forms import CourseCreateForm
from courseaffils.models import Course, CourseAccess
from django.db.models import Q
from django.shortcuts import render_to_response, get_object_or_404
from django.views.generic.edit import CreateView
from django.views.generic.list import ListView
from django.utils import timezone
from django.utils.http import urlquote
from django.http import HttpResponseForbidden, HttpResponse
from django.contrib.auth.models import User
from django.template import RequestContext


SESSION_KEY = 'ccnmtl.courseaffils.course'


def get_courses_for_user(user):
    courses = Course.objects.none()
    if user.is_staff:
        courses = Course.objects.all()
    elif not user.is_anonymous:
        courses = Course.objects.filter(group__user=user)
    return courses.order_by('-info__year', '-info__term', 'title')


def get_courses_for_instructor(user):
    courses = Course.objects.none()
    if user.is_staff:
        courses = Course.objects.all()
    elif not user.is_anonymous:
        courses = Course.objects.filter(faculty_group__user=user)

    courses = courses.order_by('-info__year', '-info__term', 'title')
    return courses.select_related(
            'info', 'group', 'faculty_group', 'settings').prefetch_related(
                'coursedetails_set')


def filter_past_courses(all_courses, current_term, current_year):
    """
    Given a queryset of courses, return only the ones in past
    semesters.
    """
    return all_courses.filter(
        Q(info__year__lt=current_year) |
        (Q(info__year=current_year) & Q(info__term__lt=current_term)))


def filter_current_courses(all_courses, current_term, current_year):
    """
    Given a queryset of courses, return only the ones in the
    current semester.
    """
    return all_courses.filter(
        info__year=current_year,
        info__term=current_term)


def filter_future_courses(all_courses, current_term, current_year):
    """
    Given a queryset of courses, return only the ones in
    future semesters.
    """
    return all_courses.filter(
        Q(info__year__gt=current_year) |
        (Q(info__year=current_year) & Q(info__term__gt=current_term)))


def filter_sandbox_courses(all_courses):
    """
    Return all sandbox courses.
    Sandbox courses don't have an associated CourseInfo.
    """
    return all_courses.filter(
        Q(info=None) | Q(info__term=None) | Q(info__year=None))


class CourseListView(ListView):
    model = Course
    view_options = [
        'past',
        'current',
        'future',
        'sandbox',
    ]

    def get_queryset(self):
        self.courses = get_courses_for_user(self.request.user)
        semester_view = self.request.GET.get('semester_view')
        current_term = get_current_term()
        current_year = timezone.now().year

        if semester_view == 'past':
            qs = filter_past_courses(
                self.courses, current_term, current_year)
        elif semester_view == 'future':
            qs = filter_future_courses(
                self.courses, current_term, current_year)
        elif semester_view == 'sandbox':
            qs = filter_sandbox_courses(self.courses)
        else:
            qs = filter_current_courses(
                self.courses, current_term, current_year)
        return qs.select_related(
            'info', 'group', 'faculty_group', 'settings').prefetch_related(
                'coursedetails_set')

    def get_context_data(self, **kwargs):
        context = super(CourseListView, self).get_context_data(**kwargs)
        semester_view = self.request.GET.get('semester_view', 'current')

        # query all the users's roles to speed rendering
        if self.request.user.is_anonymous:
            as_instructor = Course.objects.none()
            as_student = Course.objects.none()
        else:
            as_instructor = Course.objects.filter(
                faculty_group__user=self.request.user)
            as_student = Course.objects.filter(
                group__user=self.request.user).exclude(pk__in=as_instructor)

        # If semester_view isn't valid, always fall back to current.
        if semester_view not in self.view_options:
            semester_view = 'current'

        next_redirect = ''
        if 'QUERY_STRING' in self.request.META \
           and 'unset_course' not in self.request.GET:
            # just GET (until someone complains)
            escaped_path = urlquote(self.request.get_full_path())
            next_redirect = '&next=' + escaped_path

        context.update({
            'add_privilege': self.request.user.is_staff,
            'semester_view': semester_view,
            'next_redirect': next_redirect,
            'as_student': as_student,
            'as_instructor': as_instructor
        })
        return context


def select_course(request):
    available_courses = get_courses_for_user(request.user)

    list_all_link = True
    if 'list_all' not in request.GET:
        current_year = timezone.now().year
        available_courses = available_courses.exclude(
            info__year__lt=current_year)
        list_all_link = False

    current_courses = filter_current_courses(available_courses)

    response_dict = {
        'request': request,
        'user': request.user,
        'add_privilege': request.user.is_staff,
        'current_courses': current_courses,
        'courses': available_courses,
        'list_all_link': list_all_link,
    }

    if len(available_courses) == 0 and not request.user.is_staff:
        return render_to_response('courseaffils/no_courses.html',
                                  response_dict)

    response_dict['next_redirect'] = ''
    if 'QUERY_STRING' in request.META \
            and 'unset_course' not in request.GET:
        # just GET (until someone complains)
        response_dict['next_redirect'] = '&next=%s' % (
            urlquote(request.get_full_path()))

    return render_to_response('courseaffils/select_course.html',
                              response_dict,
                              context_instance=RequestContext(request))


def course_list_query(request):
    if not CourseAccess.allowed(request):
        return HttpResponseForbidden('{"error":"server not whitelisted"}')

    username = request.POST.get('user', request.GET.get('user', 'none'))
    user = get_object_or_404(User, username=username)

    courses = get_courses_for_user(user)
    data = {'courses': dict(
            [(c.group.name, {'title': c.title, })
             for c in courses]
            )}
    return HttpResponse(
        json.dumps(data, indent=2),
        content_type='application/json')


class CourseCreateView(CreateView):
    model = Course
    form_class = CourseCreateForm
    success_url = '/'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('courseaffils', '0002_auto_20160316_1223'),
    ]

    operations = [
        migrations.CreateModel(
            name='Affil',
            fields=[
                ('id',
                 models.AutoField(
                     verbose_name='ID',
                     serialize=False,
                     auto_created=True,
                     primary_key=True)),
                ('activated', models.BooleanField(default=False)),
                ('name', models.TextField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='affil',
            unique_together=set([('name', 'user')]),
        ),
    ]



from __future__ import unicode_literals

from django.http import HttpResponseRedirect
from django.conf import settings
from django.utils.encoding import smart_text

from courseaffils.models import Course, CourseAccess
from courseaffils.views import CourseListView
from courseaffils.lib import AUTO_COURSE_SELECT, is_faculty
from django.contrib.contenttypes.models import ContentType
from django.utils.deprecation import MiddlewareMixin

try:
    from django.urls import resolve, Resolver404
except ImportError:
    from django.core.urlresolvers import resolve, Resolver404

from django.shortcuts import get_object_or_404

STRUCTURED_COLLABORATION_AVAILABLE = False
try:
    from structuredcollaboration.models import Collaboration
    STRUCTURED_COLLABORATION_AVAILABLE = True
except ImportError:
    pass

SESSION_KEY = 'ccnmtl.courseaffils.course'


def has_anonymous_path(current_path):
    if hasattr(settings, "ANONYMOUS_PATHS"):
        for path in settings.ANONYMOUS_PATHS:
            if isinstance(path, str) and current_path.startswith(path):
                return True

    return False


def has_courseaffils_path(current_path):
    if hasattr(settings, 'COURSEAFFILS_PATHS'):
        for path in settings.COURSEAFFILS_PATHS:
            if isinstance(path, str) and current_path.startswith(path):
                return True
            elif hasattr(path, 'match') and path.match(current_path):
                # regex
                return True

    return False


def is_anonymous_path(current_path):
    if has_anonymous_path(current_path):
        return True

    if has_courseaffils_path(current_path):
        return False

    if not hasattr(settings, 'COURSEAFFILS_EXEMPT_PATHS'):
        return False

    for exempt_path in settings.COURSEAFFILS_EXEMPT_PATHS:
        try:
            if current_path.startswith(exempt_path):
                return True
        except TypeError:  # it wasn't a string object .. must be a regex
            if exempt_path.match(current_path):
                return True

    # if whitelist, then default is to anonymous path
    return hasattr(settings, 'COURSEAFFILS_PATHS')


def already_selected_course(request):
    return SESSION_KEY in request.session


class CourseManagerMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        if 'ANONYMIZE' in request.COOKIES:
            for user, uid in getattr(request, 'scrub_names', {}).items():
                if len(user.last_name) > 3:
                    response.content = smart_text(
                        response.content,
                        errors='ignore').replace(
                            user.get_full_name(), 'User Name_%d' % uid)
        return response

    @staticmethod
    def decorate_request(request, course):
        request.course = course
        request.coursename = course.title

        # these will show up in Error emails as part of WSGI object
        request.environ['django_username'] = request.user.username
        request.environ['django_course'] = course.title

        if STRUCTURED_COLLABORATION_AVAILABLE:
            (request.collaboration_context,
                created) = Collaboration.objects.get_or_create(
                    content_type=ContentType.objects.get_for_model(Course),
                    object_pk=str(course.pk))
            if created or request.collaboration_context.slug is None:
                request.collaboration_context.title = course.title
                request.collaboration_context.group = course.group

                for i in range(2):
                    slug_try = course.slug(attempt=i)
                    if not Collaboration.objects.filter(
                            slug=slug_try).exists():
                        request.collaboration_context.slug = slug_try
                        break
                request.collaboration_context.save()

    def course_list_view(self, request, override_view=None):
        if override_view is not None:
            response = override_view.as_view()(request)
        else:
            response = CourseListView.as_view()(request)

        if response.status_code == 200:
            response.render()

        return response

    def process_request(self, request, override_view=None):
        request.course = None  # must be present to be a caching key

        if is_anonymous_path(request.path) or \
           (
               not request.user.is_authenticated and
               not CourseAccess.allowed(request)
           ):
            return None

        if 'unset_course' in request.GET and SESSION_KEY in request.session:
            del request.session[SESSION_KEY]

        set_course = request.POST.get('set_course',
                                      request.GET.get('set_course', None))
        if set_course is not None:
            course = get_object_or_404(
                Course,
                group__name=set_course)
            if request.user.is_staff or \
               CourseAccess.allowed(request) or \
               (request.user in course.members):
                request.session[SESSION_KEY] = course
                self.decorate_request(request, course)

                if 'next' in request.GET:
                    return HttpResponseRedirect(request.GET['next'])

                return None

        if SESSION_KEY in request.session:
            course = request.session[SESSION_KEY]
            self.decorate_request(request, course)
            return None

        available_courses = Course.objects.filter(group__user=request.user)
        chosen_course = None
        try:
            requested_view, view_args, view_kwargs = resolve(
                request.get_full_path())
        except Resolver404:
            requested_view = None

        # Staff and instructors should always get the opportunity to
        # pick a course.
        if requested_view in AUTO_COURSE_SELECT:
            chosen_course = AUTO_COURSE_SELECT[requested_view](
                *view_args, **view_kwargs)
        elif (available_courses.count() == 1 and
              not request.user.is_staff and
              not is_faculty(request.user)):
            chosen_course = available_courses.first()

        if chosen_course and \
           (chosen_course in available_courses or request.user.is_staff):
            request.session[SESSION_KEY] = chosen_course
            self.decorate_request(request, chosen_course)
            return None

        return self.course_list_view(request, override_view)



from __future__ import unicode_literals

from django.contrib.auth.models import User

from courseaffils.tests.factories import UserFactory


class LoggedInSuperuserTestMixin(object):
    def setUp(self):
        self.u = User.objects.create_superuser(
            'superuser', 'admin@example.com', 'test')
        login = self.client.login(username='superuser', password='test')
        assert(login is True)


class LoggedInFacultyTestMixin(object):
    def setUp(self):
        self.u = UserFactory(username='test_faculty')
        self.u.set_password('test')
        self.u.save()
        login = self.client.login(username='test_faculty',
                                  password='test')
        assert(login is True)


class LoggedInStudentTestMixin(object):
    def setUp(self):
        self.u = UserFactory(username='test_student')
        self.u.set_password('test')
        self.u.save()
        login = self.client.login(username='test_student',
                                  password='test')
        assert(login is True)



from setuptools import setup, find_packages

version = '2.2.0'


setup(name='django-courseaffils',
      version=version,
      description="course affiliations",
      long_description="a django app which manages course information",
      classifiers=[],
      keywords='',
      author='Columbia University\'s Center for Teaching and Learning',
      author_email='ccnmtl-dev@columbia.edu',
      url='https://github.com/ccnmtl/django_courseaffils/',
      license='',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      dependency_links=[
        ],
      install_requires=[
        "Django",
      ],
      entry_points="""
      """,
      )



from __future__ import unicode_literals

from django.test import TestCase
from django.template import TemplateDoesNotExist
from courseaffils.lib import (
    faculty_courses_for_user,
    users_in_course, in_course,
    in_course_or_404,
    handle_public_name, get_public_name,
)
from courseaffils.models import Course
from django.contrib.auth.models import Group, User


class DummyRequest(object):
    COOKIES = dict()


class LibsSimpleTest(TestCase):
    def setUp(self):
        self.student_group = Group.objects.create(name="studentgroup")
        self.faculty_group = Group.objects.create(name="facultygroup")
        self.student = User.objects.create(username="student")
        self.faculty = User.objects.create(username="faculty")
        self.student.groups.add(self.student_group)
        self.faculty.groups.add(self.faculty_group)
        self.c = Course.objects.create(
            group=self.student_group,
            title="test course",
            faculty_group=self.faculty_group)

    def tearDown(self):
        self.c.delete()
        self.student_group.delete()
        self.faculty_group.delete()
        self.student.delete()
        self.faculty.delete()

    def test_users_in_course(self):
        self.assertIn(self.student, users_in_course(self.c))

    def test_in_course(self):
        self.assertTrue(in_course(self.student, self.c))
        self.assertTrue(in_course(self.student, self.student_group))
        self.assertFalse(in_course(self.faculty, self.student_group))

    def test_in_course_or_404(self):
        self.assertTrue(in_course_or_404(self.student, self.c))
        self.assertTrue(in_course_or_404(self.student, self.student_group))
        with self.assertRaises(TemplateDoesNotExist):
            # expect it to raise an error
            in_course_or_404(self.faculty, self.student_group)

    def test_handle_public_name(self):
        r = DummyRequest()
        self.assertEqual(handle_public_name(self.student, r), "student")
        r.COOKIES['ANONYMIZE'] = True
        self.assertTrue(
            handle_public_name(self.student, r).startswith("User Name_"))

    def test_get_public_name(self):
        r = DummyRequest()
        self.assertEqual(get_public_name(self.student, r), "student")

        self.assertEqual(get_public_name([self.student, ], r), "student")

    def test_faculty_courses_for_user(self):
        faculty_courses = faculty_courses_for_user(self.faculty)
        student_courses = faculty_courses_for_user(self.student)

        self.assertEqual(faculty_courses.count(), 1)
        self.assertEqual(faculty_courses.first(), self.c)
        self.assertEqual(student_courses.count(), 0)
        self.assertEqual(student_courses.first(), None)



from __future__ import unicode_literals

from django import forms
from django.test import TestCase, override_settings
from django.contrib.auth.models import Group, User
from courseaffils.columbia import CourseStringMapper
from courseaffils.forms import CourseAdminForm
from courseaffils.models import Course


@override_settings(COURSEAFFILS_COURSESTRING_MAPPER=CourseStringMapper)
class FormsSimpleTest(TestCase):
    def setUp(self):
        self.student_group = Group.objects.create(name="studentgroup")
        self.faculty_group = Group.objects.create(name="facultygroup")

        self.student = User.objects.create(username="student",
                                           last_name="long enough")
        self.student.groups.add(self.student_group)

        # faculty are part of both student & faculty groups
        self.faculty = User.objects.create(username="faculty")
        self.faculty.groups.add(self.student_group)
        self.faculty.groups.add(self.faculty_group)

        self.c = Course.objects.create(
            group=self.student_group,
            title="test course",
            faculty_group=self.faculty_group)

    def test_init(self):
        CourseAdminForm()
        CourseAdminForm(instance=self.c)

    def test_clean(self):
        s2 = Group.objects.create(name="studentgroup2")
        f = CourseAdminForm(
            dict(title="foo",
                 group=s2.id,
                 faculty_group=self.faculty_group.id,
                 add_user='student\n*nonexistant:foo'
                 ))
        f.save(commit=False)
        f.clean()
        f.clean_users_to_remove()

        self.assertTrue(s2 in self.student.groups.all())
        self.assertFalse(self.faculty_group in self.student.groups.all())

        user = User.objects.filter(username='nonexistant').first()
        self.assertIsNotNone(user)
        self.assertTrue(s2 in user.groups.all())
        self.assertTrue(self.faculty_group in user.groups.all())

    def test_clean_no_group(self):
        f = CourseAdminForm(
            dict(title="foo",
                 faculty_group=self.faculty_group.id,
                 add_user='student\n*nonexistant:foo')
        )

        with self.assertRaisesRegexp(
                ValueError,
                'The Course could not be created'):
            f.save(commit=False)

        with self.assertRaisesRegexp(
                forms.ValidationError,
                'You must select a group or enter a course string'):
            f.clean()

    def test_clean_remove(self):
        f = CourseAdminForm(instance=self.c)
        f.cleaned_data = dict(pk=self.c.id,
                              title="test course",
                              group=self.student_group.id,
                              faculty_group=self.faculty_group.id,
                              add_user='',
                              users_to_remove=[self.faculty, self.student]
                              )
        f.save(commit=False)
        f.clean()
        f.clean_users_to_remove()

        self.assertFalse(self.student_group in self.student.groups.all())

        self.assertFalse(self.student_group in self.faculty.groups.all())
        self.assertFalse(self.faculty_group in self.faculty.groups.all())



from __future__ import unicode_literals

# we need to make sure that the registration code
# gets imported, so we import it from models.py
# which will definitely be imported if the app
# is installed.

from functools import reduce
from django.db import models
from django.contrib.auth.models import User, Group
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
import re
from django.conf import settings
from courseaffils.utils import get_current_term


@python_2_unicode_compatible
class Course(models.Model):
    is_course = True
    group = models.OneToOneField(Group, on_delete=models.CASCADE)
    title = models.CharField(max_length=1024)
    faculty_group = models.ForeignKey(
        Group,
        on_delete=models.SET_NULL,
        null=True, blank=True,
        related_name='faculty_of')
    created_at = models.DateTimeField(auto_now_add=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        return self.title

    def save(self, *args, **kw):
        new = (not self.pk)
        models.Model.save(self, *args, **kw)

        if new and hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER'):
            if hasattr(settings.COURSEAFFILS_COURSESTRING_MAPPER, 'on_create'):
                settings.COURSEAFFILS_COURSESTRING_MAPPER.on_create(self)

    @property
    def members(self):
        return self.group.user_set.all().order_by(
            'first_name', 'last_name', 'username')

    @property
    def students(self):
        members = self.group.user_set.all()
        if not self.faculty_group:
            return members
        else:
            ids = self.faculty_group.user_set.values_list('id', flat=True)
            return members.exclude(id__in=ids)

    @property
    def faculty(self):
        if self.faculty_group:
            return self.faculty_group.user_set.all()
        else:
            return tuple()

    @property
    def user_set(self):
        if self.group_id:
            return self.group.user_set
        else:
            return None

    @property
    def faculty_filter(self):
        return reduce(lambda x, y: x | y,  # composable Q's
                      [models.Q(author=f) for f in self.faculty],
                      models.Q(author=-1)  # impossible
                      ) & models.Q(course=self)

    def is_faculty(self, user):
        return user.is_staff or (
            self.faculty_group and
            self.faculty_group.user_set.filter(id=user.id).exists())

    def is_member(self, user):
        return (user.is_staff or
                self.group.user_set.filter(id=user.id).exists())

    def is_true_member(self, user):
        return self.group.user_set.filter(id=user.id).exists()

    def is_true_faculty(self, user):
        return self.faculty_group and (
            self.faculty_group.user_set.filter(id=user.id).exists())

    def default_slug(self, **kw):
        return re.sub(r'\W', '', re.sub(' ', '_', self.title))

    def slug(self, **kw):
        if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER'):
            if hasattr(settings.COURSEAFFILS_COURSESTRING_MAPPER,
                       'course_slug'):
                return settings.COURSEAFFILS_COURSESTRING_MAPPER.course_slug(
                    self, **kw)

        return self.default_slug(**kw)

    def details(self):
        return dict(
            [(i.name, i) for i in self.coursedetails_set.all()]
        )

    def get_detail(self, name, default):
        try:
            return CourseDetails.objects.get(course=self, name=name).value
        except CourseDetails.DoesNotExist:
            return default

    def add_detail(self, name, value):
        CourseDetails.objects.update_or_create(
            course=self, name=name, defaults={'value': value})


@python_2_unicode_compatible
class CourseSettings(models.Model):
    course = models.OneToOneField(
        Course,
        on_delete=models.CASCADE,
        related_name='settings')
    custom_headers = models.TextField(
        blank=True, null=True,
        help_text=("""Replaces main.css link in header.  """
                   """You need to add this as full HTML (&lt;link """
                   """rel="stylesheet" href="...." />) but the """
                   """advantage is you can add custom javascript here, too.""")
    )

    def __str__(self):
        return 'Settings for %s' % self.course.title

    class Meta:
        verbose_name_plural = 'Course Settings'


@python_2_unicode_compatible
class CourseInfo(models.Model):
    term_choices = {1: 'Spring', 2: 'Summer', 3: 'Fall'}

    course = models.OneToOneField(
        Course,
        on_delete=models.CASCADE,
        related_name='info')

    year = models.IntegerField(null=True, blank=True)
    term = models.IntegerField(
        null=True, blank=True, choices=term_choices.items())

    # for ability to query what courses are going on NOW
    starttime = models.TimeField(null=True, blank=True)
    endtime = models.TimeField(null=True, blank=True)
    days = models.CharField(
        max_length=7, null=True, blank=True,
        help_text='e.g. "MTWRF"')

    def time(self):
        return '%s%s' % (
            self.days if self.days else '',
            ' %s-%s' % (self.starttime.strftime('%H:%M'),
                        self.endtime.strftime('%H:%M')
                        ) if self.starttime else ''
        )

    def termyear(self):
        term = self.term_choices.get(self.term, '')
        if self.year:
            term = term + (' %d' % self.year)
        return term

    def display(self):
        return '%s %s %s-%s' % (self.termyear(),
                                self.days, self.starttime, self.endtime)

    def __str__(self):
        return '%s (%s) %s %s-%s' % (self.course.title,
                                     self.termyear(),
                                     self.days, self.starttime, self.endtime)

    class Meta:
        verbose_name_plural = 'Course Info'


@python_2_unicode_compatible
class CourseDetails(models.Model):
    """useful for storing info like 'semester', 'url' """
    course = models.ForeignKey(Course, on_delete=models.CASCADE)
    name = models.CharField(
        max_length=64,
        help_text=("""type of data. Useful ones are """
                   """'instructor', 'semester', 'url', """
                   """'campus', 'times', 'call_number'"""))
    value = models.CharField(max_length=1024,
                             help_text="""The name's value for the course.""")

    def __str__(self):
        return '(%s) %s: %s' % (self.course.title, self.name, self.value)

    class Meta:
        verbose_name_plural = 'Course Details'


# server2server admin access support
class CourseAccess:
    @classmethod
    def allowed(cls, request):
        secret = request.POST.get('secret', request.GET.get('secret', None))
        return (secret
                in getattr(settings, 'SERVER_ADMIN_SECRETKEYS', {}).values()
                )

    @classmethod
    def respond(cls, message):
        pass


@python_2_unicode_compatible
class Affil(models.Model):
    """Model for storing activatable affiliations.

    Faculty use this to 'activate' it into a Group/Course.
    """
    class Meta:
        unique_together = ('name', 'user')

    is_affil = True
    activated = models.BooleanField(default=False)
    name = models.TextField()
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    def to_dict(self):
        if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER'):
            return settings.COURSEAFFILS_COURSESTRING_MAPPER.to_dict(
                self.name)
        else:
            return None

    @property
    def is_faculty(self):
        """Returns True if this affil is a faculty affil."""
        d = self.to_dict()
        return d.get('member') == 'fc'

    @property
    def past_present_future(self):
        """Find out if this Affil is in the past, present, or future.

        Returns -1 if this Affil is in the past, 0 if in the present,
        and 1 if in the future. If there was a parse error, returns None.
        """
        current_year = timezone.now().year
        current_term = get_current_term()

        affil_dict = self.to_dict()

        if affil_dict is None:
            return None

        try:
            year = int(affil_dict.get('year'))
        except TypeError:
            return None

        try:
            term = int(affil_dict.get('term'))
        except TypeError:
            return None

        if year == current_year and term == current_term:
            return 0
        elif year < current_year or (
                year == current_year and term < current_term):
            return -1
        else:
            return 1

    @property
    def courseworks_name(self):
        """Returns the Courseworks formatted name.

        e.g.: CUcourse_NURSN6610_001_2008_2

        If no mapper is configured, returns None.
        """
        if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER') and \
           settings.COURSEAFFILS_COURSESTRING_MAPPER is not None:
            affil_dict = self.to_dict()
            return 'CUcourse_{}{}{}_{}_{}_{}'.format(
                affil_dict['dept'].upper(),
                affil_dict['letter'],
                affil_dict['number'],
                affil_dict['section'],
                affil_dict['year'],
                affil_dict['term'])
        return None

    @property
    def coursedirectory_name(self):
        """Returns the Course Directory formatted name.

        e.g.: 20082NURS6610N001

        If no mapper is configured, returns None.
        """
        if hasattr(settings, 'COURSEAFFILS_COURSESTRING_MAPPER') and \
           settings.COURSEAFFILS_COURSESTRING_MAPPER is not None:
            affil_dict = self.to_dict()
            return '{}{}{}{}{}{}'.format(
                affil_dict['year'],
                affil_dict['term'],
                affil_dict['dept'].upper(),
                affil_dict['number'],
                affil_dict['letter'].upper(),
                affil_dict['section'])
        return None

    @property
    def shortname(self):
        affil_dict = self.to_dict()
        if affil_dict is not None:
            return affil_dict['dept'].upper() + affil_dict['number']

    def get_course(self):
        """Returns an associated course for this affil, if it exists."""
        fgroup = Group.objects.filter(name=self.name).first()
        if Course.objects.filter(faculty_group=fgroup).exists():
            return Course.objects.filter(faculty_group=fgroup).first()

        studentaffil = re.sub(r'\.fc\.', '.st.', self.name)
        sgroup = Group.objects.filter(name=studentaffil).first()
        if Course.objects.filter(group=sgroup).exists():
            return Course.objects.filter(group=sgroup).first()

        return None
